use dioxus::prelude::*;

use crate::modrinth::tags::game_versions::{get_game_versions, GameVersion};
use crate::Route;

#[component]
pub fn New() -> Element {
    let nav = navigator();

    let game_versions = use_resource(|| async move {
        let response = get_game_versions().await;

        match response {
            Ok(r) => Ok(r
                .into_iter()
                .filter(|v| v.major == true)
                .collect::<Vec<GameVersion>>()),
            Err(err) => Err(err),
        }
    });

    let submit = move |event: FormEvent| async move {
        if let Ok(modpack_id) = create_modpack(
            event.values()["name"].as_value(),
            event.values()["game_version"].as_value(),
            event.values()["modloader"].as_value(),
        )
        .await
        {
            nav.push(Route::Modpack { id: modpack_id });
        }
    };

    rsx! {
        div {
            class: "max-w-md w-full mx-auto",
            h2 { class: "text-2xl font-bold mt-20", "Create a new modpack" }
            form {
                class: "mt-10",
                onsubmit: submit,
                input { r#type: "text", name: "name", placeholder: "Name", class: "input input-bordered w-full" }
                div {
                    class: "flex gap-5 mt-6",
                    select {
                        class: "select select-bordered w-full",
                        name: "game_version",
                        option { disabled: true, selected: true, "Minecraft Version" },
                        match &*game_versions.read_unchecked() {
                            Some(Ok(response)) => rsx! {
                                for game_version in response {
                                    option { "{game_version.version}" }
                                }
                            },
                            _ => rsx! { option { disabled: true, "Loading..." } }
                        }
                    }
                    select {
                        class: "select select-bordered",
                        name: "modloader",
                        option { disabled: true, selected: true, "Modloader" },
                        option { "Fabric" },
                        option { "Quilt" },
                        option { "Forge" },
                        option { "Neoforge" },
                    }
                }
                button { class: "btn btn-primary mt-10 w-full", "Submit" }
            }
        }
    }
}

#[server]
async fn create_modpack(
    name: String,
    game_version: String,
    modloader: String,
) -> Result<String, ServerFnError> {
    use crate::server::auth::User;
    use crate::server::modpack::Modpack;

    let user = User::from_cookie().await?;

    let modpack = Modpack::new(name, game_version, modloader, user.id).await?;

    Ok(modpack.id)
}
