#[cfg(feature = "server")]
use super::surrealdb::DB;
use dioxus::prelude::*;
use serde::{Deserialize, Serialize};
#[cfg(feature = "server")]
use surrealdb::sql::Thing;

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct User {
    pub id: String,
    pub email: String,
    pub password: Option<String>,
}

#[cfg(feature = "server")]
#[derive(Debug, Deserialize)]
struct Record {
    id: Thing,
}

#[derive(Debug, Deserialize)]
struct Session {
    user_id: String,
    id: String,
    expires: i64,
    token: String,
}

impl User {
    #[cfg(feature = "server")]
    pub async fn from_cookie() -> Result<Self, ServerFnError> {
        use axum_extra::extract::CookieJar;

        let jar: CookieJar = extract().await?;

        let session_token = jar
            .get("session_token")
            .ok_or_else(|| ServerFnError::new("Session token cookie is not set"))?;

        let session_token = session_token.value();

        let mut res = DB
            .query("SELECT type::string(user.id) as id, user.email as email FROM session WHERE token = $session_token")
            .bind(("session_token", session_token))
            .await?;

        let user: Option<User> = res.take(0)?;

        match user {
            Some(u) => {
                tracing::info!("Authorized session for {}", u.id);
                Ok(u)
            }
            None => Err(ServerFnError::ServerError(
                "Could not authorize session".to_string(),
            )),
        }
    }
}

impl Session {
    #[cfg(feature = "server")]
    async fn create(user_id: &String) -> Result<Self, ServerFnError> {
        // Create a session
        let mut res = DB
            .query("CREATE session SET user = type::thing($user_id) RETURN type::string(id) as id, type::string(id) as user_id, time::unix(expires) as expires, token;")
            .bind(("user_id", user_id))
            .await?;

        let session: Option<Session> = res.take(0)?;

        match session {
            Some(s) => {
                tracing::info!("Created new session for {}", user_id);
                Ok(s)
            }
            None => Err(ServerFnError::ServerError(
                "Could not generate session".to_string(),
            )),
        }
    }

    #[cfg(feature = "server")]
    fn write(&self) -> Result<(), ServerFnError> {
        use axum::http::{header, HeaderValue};
        use axum_extra::extract::cookie::{Cookie, SameSite};

        let mut cookie = Cookie::new("session_token", &self.token);

        cookie.set_same_site(SameSite::Strict);

        server_context()
            .response_parts_mut()
            .unwrap()
            .headers
            .insert(
                header::SET_COOKIE,
                HeaderValue::from_str(&cookie.to_string())?,
            );

        Ok(())
    }
}

#[server(Register)]
pub async fn register(email: String, password: String) -> Result<String, ServerFnError> {
    let mut res = DB
        .query("CREATE user SET email = $email, password = crypto::argon2::generate($password)")
        .bind(("email", email))
        .bind(("password", password))
        .await?;

    let user: Option<Record> = res.take(0)?;

    match user {
        Some(Record { id }) => {
            tracing::info!("Created new user ({id})");

            let session = Session::create(&id.to_raw()).await?;
            session.write()?;

            Ok(id.to_string())
        }
        _ => Err(ServerFnError::ServerError("Could not get id".to_string())),
    }
}

#[server(Signin)]
pub async fn signin(email: String, password: String) -> Result<User, ServerFnError> {
    // Find the user with the correct email and password
    let mut res = DB
        .query("SELECT type::string(id) as id, email, password FROM user WHERE email = $email AND crypto::argon2::compare(password, $password)")
        .bind(("email", email))
        .bind(("password", password))
        .await?;

    let user: Option<User> = res.take(0)?;

    match user {
        Some(u) => {
            tracing::info!("User ({}) has signed in", u.email);

            let session = Session::create(&u.id).await?;
            session.write()?;

            Ok(u)
        }
        _ => Err(ServerFnError::ServerError("Could not get id".to_string())),
    }
}

#[server(GetUser)]
pub async fn get_current_user() -> Result<User, ServerFnError> {
    User::from_cookie().await
}
