use dioxus::prelude::*;
use serde::{Deserialize, Serialize};

#[cfg(feature = "server")]
use super::surrealdb::DB;

#[derive(Debug, Serialize, Deserialize, PartialEq, Clone)]
pub struct Modpack {
    pub id: String,
    pub owner_id: String,
    pub name: String,
    pub game_version: String,
    pub modloader: String,
    pub projects: Vec<Project>,
}

#[derive(Debug, Serialize, Deserialize, PartialEq, Clone)]
pub struct Project {
    pub project_id: String,
    pub version_id: String,
}

impl Modpack {
    #[cfg(feature = "server")]
    pub async fn new(
        name: String,
        game_version: String,
        modloader: String,
        user_id: String,
    ) -> Result<Self, ServerFnError> {
        let mut res = DB
            .query("CREATE modpack SET name = $name, owner = type::thing($user_id), game_version = $game_version, modloader = $modloader, projects = [] RETURN type::string(id) as id, type::string(owner) as owner_id, game_version, modloader, name, projects")
            .bind(("name", name))
            .bind(("user_id", user_id))
            .bind(("game_version", game_version))
            .bind(("modloader", modloader))
            .await?;

        let modpack: Option<Modpack> = res.take(0)?;

        match modpack {
            Some(m) => {
                tracing::info!("Created new modpack: {}", m.id);
                Ok(m)
            }
            None => Err(ServerFnError::ServerError(
                "Could not create modpack".to_string(),
            )),
        }
    }

    #[cfg(feature = "server")]
    pub async fn get_all(user_id: String) -> Result<Vec<Self>, ServerFnError> {
        let mut res = DB
            .query("SELECT type::string(id) as id, type::string(owner) as owner_id, name, modloader, game_version, projects FROM modpack WHERE type::string(owner) = $owner_id;")
            .bind(("owner_id", user_id))
            .await?;

        let modpacks: Vec<Self> = res.take(0)?;

        Ok(modpacks)
    }

    #[cfg(feature = "server")]
    pub async fn get(id: String) -> Result<Self, ServerFnError> {
        let mut res = DB
            .query("SELECT type::string(id) as id, type::string(owner) as owner_id, name, modloader, game_version, projects FROM modpack WHERE type::string(id) = $id;")
            .bind(("id", id))
            .await?;

        let modpack: Option<Self> = res.take(0)?;

        match modpack {
            Some(m) => Ok(m),
            None => Err(ServerFnError::ServerError(
                "Could not get modpack".to_string(),
            )),
        }
    }
}
